﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=9.3
@EndOfDesignText@
'v.0.13 (c) Vlad Pomelov aka Peacemaker
'Class for trend\chart analysis
Private Sub Class_Globals
	Private dbName As String
	Private const dbTable As String = "trend_table"
	'Private const dbOperTable As String = "operating_table"
	Private mSQL As SQL
	Private mInitialized As Boolean
	Private OverallAVG As Double    'average Y-value of the whole period
End Sub

'Initializes the object.
Public Sub Initialize
	Dim dbName As String = "trend_db_" & DateTime.Now & ".sqlite"
	File.Delete(Folder, dbName)	'deleted old database for clean test
	If mSQL.IsInitialized = False Then
		mSQL.InitializeSQLite(Folder, dbName, True)
	End If
	
	Dim ft As Map
	ft.Initialize
	ft.Put("id", DBUtils.DB_INTEGER)
	ft.Put("x", DBUtils.DB_TEXT)
	ft.Put("y", DBUtils.DB_REAL)
	DBUtils.CreateTable(mSQL, dbTable, ft, "id")
	mInitialized = True
End Sub

'Set the data set period: map key = X abscissa, map value = Y ordinate value
'Example:
'Dim SourceMap As Map = CreateMap("Temperature 10:00":10, "Temperature 10:10":13, "Temperature 10:20":16, "Temperature 10:30":19, "Temperature 14:00":30)
Public Sub DataMapToTrend(SourceMap As Map)
	DBUtils.ClearTable(mSQL, dbTable)
	Dim L As List:L.Initialize
	For i = 0 To SourceMap.Size - 1
		Dim ft As Map
		ft.Initialize
		ft.Put("x", SourceMap.GetKeyAt(i))
		ft.Put("y", SourceMap.GetValueAt(i))
		L.Add(ft)
	Next	
	DBUtils.InsertMaps(mSQL, dbTable, L)
	OverallAVG = DBUtils.GetStringFromSQL(mSQL, "SELECT AVG(y) FROM " & dbTable)
End Sub

Public Sub AverageY As Double
	Return OverallAVG
End Sub

'True is the chart is of rising character
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub isRisingY (StartPercent As Int, StopPercent As Int) As Boolean
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	Private SubIntervals_qty As Int 'the source period is divided into this number parts [ i] (iterable) to compare them
	If count > 10 Then
		SubIntervals_qty = 10
	Else
		SubIntervals_qty = count / 2
	End If
	
	StartPercent = StartPercent * count / 100
	StopPercent = StopPercent * count / 100
	Dim Range As Int = StopPercent - StartPercent
	If Range < 0 Then Log("Error: StartPercent must be smaller thwn StopPercent")
	
'	Dim ft As Map
'	ft.Initialize
'	ft.Put("id", DBUtils.DB_INTEGER)
'	ft.Put("y", DBUtils.DB_REAL)
'	DBUtils.CreateTable(mSQL, dbOperTable, ft, "id")
	Dim Interval As Int = Floor(Range / SubIntervals_qty) + 1	'qty of analysing subintervals of the whole data set
	Dim operations As List
	operations.Initialize
	For i = StartPercent To StopPercent - 1 Step Interval
		Dim upto As Int = i + Interval
		Dim req As String = "SELECT AVG(y) FROM " & dbTable & " WHERE id >= " & i & " AND id < " & upto
		'Log(req)
		Dim avg As Double = DBUtils.GetStringFromSQL(mSQL, req)
		operations.Add(avg)
	Next
	Dim oper_qty As Int
	For i = 0 To operations.Size - 2
		Dim prev As Double = operations.Get(i)
		Dim nex As Double = operations.Get(i+1)
		If nex >= prev Then
			oper_qty = oper_qty + 1
		Else
			oper_qty = oper_qty - 1
		End If
	Next
	If oper_qty > operations.Size / 2 Then
		Return True	'more than half rising numbers
	Else
		Return False
	End If
End Sub

'True is the chart is of falling character
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub isFallingY (StartPercent As Int, StopPercent As Int) As Boolean
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	Private SubIntervals_qty As Int 'the source period is divided into this number parts [ i] (iterable) to compare them
	If count > 10 Then
		SubIntervals_qty = 10
	Else
		SubIntervals_qty = count / 2
	End If
	StartPercent = StartPercent * count / 100
	StopPercent = StopPercent * count / 100
	Dim Range As Int = StopPercent - StartPercent
	If Range < 0 Then Log("Error: StartPercent must be smaller thwn StopPercent")
	
	Dim Interval As Int = Floor(Range / SubIntervals_qty) + 1	'qty of analysing subintervals of the whole data set
	Dim operations As List
	operations.Initialize
	For i = StartPercent To StopPercent - 1 Step Interval
		Dim upto As Int = i + Interval
		Dim req As String = "SELECT AVG(y) FROM " & dbTable & " WHERE id >= " & i & " AND id < " & upto
		'Log(req)
		Dim avg As Double = DBUtils.GetStringFromSQL(mSQL, req)
		operations.Add(avg)
	Next
	Dim oper_qty As Int
	For i = 0 To operations.Size - 2
		Dim prev As Double = operations.Get(i)
		Dim nex As Double = operations.Get(i+1)
		If nex <= prev Then
			oper_qty = oper_qty + 1
		Else
			oper_qty = oper_qty - 1
		End If
	Next
	If oper_qty > operations.Size / 2 Then
		Return True	'more than half falling numbers
	Else
		Return False
	End If
End Sub

'True is the chart is wobbling around a middle value, not changing higher or lower strongly
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub isWobblingY (StartPercent As Int, StopPercent As Int) As Boolean
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	Private SubIntervals_qty As Int 'the source period is divided into this number parts [ i] (iterable) to compare them
	If count > 10 Then
		SubIntervals_qty = 10
	Else
		SubIntervals_qty = count / 2
	End If
	StartPercent = StartPercent * count / 100
	StopPercent = StopPercent * count / 100
	Dim Range As Int = StopPercent - StartPercent
	If Range < 0 Then Log("Error: StartPercent must be smaller thwn StopPercent")
	
	Dim Interval As Int = Floor(Range / SubIntervals_qty) + 1	'qty of analysing subintervals of the whole data set
	Dim operations As List
	operations.Initialize
	For i = StartPercent To StopPercent - 1 Step Interval
		Dim upto As Int = i + Interval
		Dim req As String = "SELECT AVG(y) FROM " & dbTable & " WHERE id >= " & i & " AND id < " & upto
		'Log(req)
		Dim avg As Double = DBUtils.GetStringFromSQL(mSQL, req)
		operations.Add(avg)
	Next
	Dim oper_qty As Int
	For i = 0 To operations.Size - 2
		Dim prev As Double = operations.Get(i)
		Dim nex As Double = operations.Get(i+1)
		If nex >= prev Then
			oper_qty = oper_qty + 1
		Else
			oper_qty = oper_qty - 1
		End If
	Next
	If oper_qty < operations.Size / 2 Then
		Return True
	Else
		Return False
	End If
End Sub

Public Sub hasMaximum
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	
End Sub

Private Sub Folder As String
	Dim res As String = File.DirApp
	Return res
End Sub

Public Sub IsInitialized As Boolean
	Return mInitialized
End Sub

Public Sub Close
	If mSQL.IsInitialized Then mSQL.Close
	File.Delete(Folder, dbName)
	mInitialized = False
End Sub

