﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=9.3
@EndOfDesignText@
'v.0.15 (c) Vlad Pomelov aka Peacemaker
'Class for trend\chart analysis
Private Sub Class_Globals
	Private dbName As String
	Private const dbTable As String = "trend_table"
	Private mSQL As SQL
	Private mInitialized As Boolean
	Private OverallAVG As Double    'average Y-value of the whole period
End Sub

'The object must be initialized first.
Public Sub Initialize
	Dim dbName As String = "trend_db_" & DateTime.Now & ".sqlite"
	File.Delete(Folder, dbName)	'deleted old database for clean test
	If mSQL.IsInitialized = False Then
		#if B4J
		mSQL.InitializeSQLite(Folder, dbName, True)
		#else if B4A
		mSQL.Initialize(Folder, dbName, True)
		#end if
	End If
	
	Dim ft As Map
	ft.Initialize
	ft.Put("id", DBUtils.DB_INTEGER)
	ft.Put("x", DBUtils.DB_TEXT)
	ft.Put("y", DBUtils.DB_REAL)
	DBUtils.CreateTable(mSQL, dbTable, ft, "id")
	mInitialized = True
End Sub

'Set the data set period: map key = X abscissa, map value = Y ordinate value
'Example:
'Dim SourceMap As Map = CreateMap("Temperature 10:00":10, "Temperature 10:10":13, "Temperature 10:20":16, "Temperature 10:30":19, "Temperature 14:00":30)
Public Sub DataMapToTrend(SourceMap As Map)
	DBUtils.ClearTable(mSQL, dbTable)
	Dim L As List:L.Initialize
	For i = 0 To SourceMap.Size - 1
		Dim ft As Map
		ft.Initialize
		ft.Put("x", SourceMap.GetKeyAt(i))
		ft.Put("y", SourceMap.GetValueAt(i))
		L.Add(ft)
	Next	
	DBUtils.InsertMaps(mSQL, dbTable, L)
	OverallAVG = DBUtils.GetStringFromSQL(mSQL, "SELECT AVG(y) FROM " & dbTable)
End Sub

Public Sub AverageY As Double
	Return OverallAVG
End Sub

'Returns maximum if the chart is of rising character
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub isRisingY (StartPercent As Int, StopPercent As Int) As String
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	StartPercent = StartPercent * count / 100
	StopPercent = StopPercent * count / 100
	Dim Range As Int = StopPercent - StartPercent
	If Range < 0 Then Log("Error: StartPercent must be smaller than StopPercent")
	count = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable & " WHERE id >= " & StartPercent & " AND id <= " & StopPercent)
	Private SubIntervals_qty As Int 'the source period is divided into this number parts [ i] (iterable) to compare them
	If count > 10 Then
		SubIntervals_qty = 10
	Else
		SubIntervals_qty = count / 3
	End If
	
	Dim Interval As Int = Floor(Range / SubIntervals_qty) + 1	'qty of analysing subintervals of the whole data set
	Dim operations As List
	operations.Initialize
	Dim MaxValue As Float = DBUtils.GetStringFromSQL(mSQL, "SELECT MIN(y) FROM " & dbTable)
	For i = StartPercent To StopPercent - 1 Step Interval
		Dim upto As Int = i + Interval
		Dim req As String = "SELECT AVG(y) FROM " & dbTable & " WHERE id >= " & i & " AND id < " & upto
		'Log(req)
		Dim avg As Double = DBUtils.GetStringFromSQL(mSQL, req)
		If avg >= MaxValue Then MaxValue = avg
		operations.Add(avg)
	Next
	
	Dim oper_qty As Int
	For i = 0 To operations.Size - 2
		Dim prev As Double = operations.Get(i)
		Dim nex As Double = operations.Get(i+1)
		If nex >= prev Then
			oper_qty = oper_qty + 1
		Else
			oper_qty = oper_qty - 1
		End If
	Next
	If oper_qty >= operations.Size / 2 Then
		Return MaxValue
	Else
		Return Null
	End If
End Sub

'Returns minimum if the chart is of falling character
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub isFallingY (StartPercent As Int, StopPercent As Int) As String
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	StartPercent = StartPercent * count / 100
	StopPercent = StopPercent * count / 100
	Dim Range As Int = StopPercent - StartPercent
	If Range < 0 Then Log("Error: StartPercent must be smaller than StopPercent")
	count = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable & " WHERE id >= " & StartPercent & " AND id <= " & StopPercent)
	Private SubIntervals_qty As Int 'the source period is divided into this number parts [ i] (iterable) to compare them
	If count > 10 Then
		SubIntervals_qty = 10
	Else
		SubIntervals_qty = count / 3
	End If
	
	Dim Interval As Int = Floor(Range / SubIntervals_qty) + 1	'qty of analysing subintervals of the whole data set
	Dim operations As List
	operations.Initialize
	Dim MinValue As Float = DBUtils.GetStringFromSQL(mSQL, "SELECT MAX(y) FROM " & dbTable)
	For i = StartPercent To StopPercent - 1 Step Interval
		Dim upto As Int = i + Interval
		Dim req As String = "SELECT AVG(y) FROM " & dbTable & " WHERE id >= " & i & " AND id < " & upto
		'Log(req)
		Dim avg As Double = DBUtils.GetStringFromSQL(mSQL, req)
		If avg <= MinValue Then MinValue = avg
		operations.Add(avg)
	Next
	Dim oper_qty As Int
	For i = 0 To operations.Size - 2
		Dim prev As Double = operations.Get(i)
		Dim nex As Double = operations.Get(i+1)
		If nex <= prev Then
			oper_qty = oper_qty + 1
		Else
			oper_qty = oper_qty - 1
		End If
	Next
	If oper_qty >= operations.Size / 2 Then
		Return MinValue
	Else
		Return Null
	End If
End Sub

'Returns average value if the chart is wobbling around a middle value.
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub isWobblingY (StartPercent As Int, StopPercent As Int) As String
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	StartPercent = StartPercent * count / 100
	StopPercent = StopPercent * count / 100
	Dim Range As Int = StopPercent - StartPercent
	If Range < 0 Then Log("Error: StartPercent must be smaller than StopPercent")
	count = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable & " WHERE id >= " & StartPercent & " AND id <= " & StopPercent)
	Private SubIntervals_qty As Int 'the source period is divided into this number parts [ i] (iterable) to compare them
	If count > 10 Then
		SubIntervals_qty = 10
	Else
		SubIntervals_qty = count / 3
	End If
	
	Dim Interval As Int = Floor(Range / SubIntervals_qty) + 1	'qty of analysing subintervals of the whole data set
	Dim operations As List
	operations.Initialize
	For i = StartPercent To StopPercent - 1 Step Interval
		Dim upto As Int = i + Interval
		Dim req As String = "SELECT AVG(y) FROM " & dbTable & " WHERE id >= " & i & " AND id < " & upto
		'Log(req)
		Dim avg As Double = DBUtils.GetStringFromSQL(mSQL, req)
		operations.Add(avg)
	Next
	Dim oper_qty As Int
	For i = 0 To operations.Size - 2
		Dim prev As Double = operations.Get(i)
		Dim nex As Double = operations.Get(i+1)
		If nex >= prev Then
			oper_qty = oper_qty + 1
		Else
			oper_qty = oper_qty - 1
		End If
	Next
	If oper_qty <= operations.Size / 2 Then
		Return OverallAVG
	Else
		Return Null
	End If
End Sub

'Returns max value if the chart has a maxumum
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub hasMaximum (StartPercent As Int, StopPercent As Int) As String
	Dim LeftRising As String = isRisingY(StartPercent, StopPercent * 0.5)
	Dim RightFalling As String = isFallingY(StopPercent * 0.5, StopPercent)
	If IsNumber(LeftRising) And IsNumber(RightFalling) Then
		Return Max(LeftRising, MaxY(StartPercent, StopPercent))
	Else
		Return Null
	End If
End Sub

'True if the chart has a minimum
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub hasMinimum (StartPercent As Int, StopPercent As Int) As String
	Dim LeftFalling As String = isFallingY(StartPercent, StopPercent * 0.5)
	Dim RightRising As String = isRisingY(StopPercent * 0.5, StopPercent)
	If IsNumber(LeftFalling) And IsNumber(RightRising) Then
		Return Min(LeftFalling, MinY(StartPercent, StopPercent))
	Else
		Return Null
	End If
End Sub

Private Sub Folder As String
	#if B4J
	Dim res As String = File.DirApp
	#else if B4A
	Dim res As String = DBUtils.GetDBFolder
	#end if
	Return res
End Sub

Public Sub IsInitialized As Boolean
	Return mInitialized
End Sub

Public Sub Close
	If mSQL.IsInitialized Then mSQL.Close
	File.Delete(Folder, dbName)
	mInitialized = False
End Sub

'Returns max of the range
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub MaxY (StartPercent As Int, StopPercent As Int) As String
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	StartPercent = StartPercent * count / 100
	StopPercent = StopPercent * count / 100
	Dim Range As Int = StopPercent - StartPercent
	If Range < 0 Then Log("Error: StartPercent must be smaller than StopPercent")
	count = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable & " WHERE id >= " & StartPercent & " AND id <= " & StopPercent)
	Private SubIntervals_qty As Int 'the source period is divided into this number parts [ i] (iterable) to compare them
	If count > 10 Then
		SubIntervals_qty = 10
	Else
		SubIntervals_qty = count / 3
	End If
	
	Dim Interval As Int = Floor(Range / SubIntervals_qty) + 1	'qty of analysing subintervals of the whole data set
	Dim operations As List
	operations.Initialize
	Dim MaxValue As Float = DBUtils.GetStringFromSQL(mSQL, "SELECT MIN(y) FROM " & dbTable)
	For i = StartPercent To StopPercent - 1 Step Interval
		Dim upto As Int = i + Interval
		Dim req As String = "SELECT MAX(y) FROM " & dbTable & " WHERE id >= " & i & " AND id < " & upto
		Dim m As Double = DBUtils.GetStringFromSQL(mSQL, req)
		If m >= MaxValue Then MaxValue = m
		operations.Add(m)
	Next
	Return MaxValue
End Sub

'Returns min of the range
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub MinY (StartPercent As Int, StopPercent As Int) As String
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	StartPercent = StartPercent * count / 100
	StopPercent = StopPercent * count / 100
	Dim Range As Int = StopPercent - StartPercent
	If Range < 0 Then Log("Error: StartPercent must be smaller than StopPercent")
	count = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable & " WHERE id >= " & StartPercent & " AND id <= " & StopPercent)
	Private SubIntervals_qty As Int 'the source period is divided into this number parts [ i] (iterable) to compare them
	If count > 10 Then
		SubIntervals_qty = 10
	Else
		SubIntervals_qty = count / 3
	End If
	
	Dim Interval As Int = Floor(Range / SubIntervals_qty) + 1	'qty of analysing subintervals of the whole data set
	Dim operations As List
	operations.Initialize
	Dim MinValue As Float = DBUtils.GetStringFromSQL(mSQL, "SELECT MAX(y) FROM " & dbTable)
	For i = StartPercent To StopPercent - 1 Step Interval
		Dim upto As Int = i + Interval
		Dim req As String = "SELECT MIN(y) FROM " & dbTable & " WHERE id >= " & i & " AND id < " & upto
		Dim m As Double = DBUtils.GetStringFromSQL(mSQL, req)
		If m <= MinValue Then MinValue = m
		operations.Add(m)
	Next
	Return MinValue
End Sub