﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=9.3
@EndOfDesignText@
'v.0.20 (c) Vlad Pomelov aka Peacemaker
'Class for trend\chart analysis
Private Sub Class_Globals
	Private dbName As String
	Private const dbTable As String = "trend_table"
	Private mSQL As SQL
	Private mInitialized As Boolean
	Private OverallAVG As Double    'average Y-value of the whole period
	'type has fields for parameters of the analyzed data range, including "stableZZ" where ZZ = percent
	Type TrendResult (MinX As String, MaxX As String, MinY As String, MaxY As String, AvgX As String, AvgY As Double, rising As Boolean, falling As Boolean, wobbling As Boolean, hasMaxY As Boolean, hasMinY As Boolean, stable As Boolean, stableWithin As Float)
	
	Public NameYaxis As String = "Temperature"	'edit in your code
	Public UnitsYaxis As String = "°C"	'edit in your code
	Public NameXaxis As String = "Time"	'edit in your code
	Public UnitsXaxis As String = "hours"	'edit in your code
End Sub

'The object must be initialized first.
Public Sub Initialize
	If dbName <> "" And mSQL.IsInitialized Then
		Close
	End If
	Dim dbName As String = "trend_db_" & DateTime.Now & ".sqlite"
	File.Delete(Folder, dbName)	'deleted old database for clean test
	If mSQL.IsInitialized = False Then
		#if B4J
		mSQL.InitializeSQLite(Folder, dbName, True)
		#else if B4A
		mSQL.Initialize(Folder, dbName, True)
		#end if
	End If
	
	Dim ft As Map
	ft.Initialize
	ft.Put("id", DBUtils.DB_INTEGER)
	ft.Put("x", DBUtils.DB_TEXT)
	ft.Put("y", DBUtils.DB_REAL)
	DBUtils.CreateTable(mSQL, dbTable, ft, "id")
	mInitialized = True
End Sub

'Set the data set period: map key = X abscissa, map value = Y ordinate value
'Example:
'Dim SourceMap As Map = CreateMap("Temperature 10:00":10, "Temperature 10:10":13, "Temperature 10:20":16, "Temperature 10:30":19, "Temperature 14:00":30)
Public Sub DataMapToTrend(SourceMap As Map)
	DBUtils.ClearTable(mSQL, dbTable)
	Dim L As List:L.Initialize
	For i = 0 To SourceMap.Size - 1
		Dim ft As Map
		ft.Initialize
		ft.Put("x", SourceMap.GetKeyAt(i))
		ft.Put("y", SourceMap.GetValueAt(i))
		L.Add(ft)
	Next	
	DBUtils.InsertMaps(mSQL, dbTable, L)
	OverallAVG = mSQL.ExecQuerySingleResult("SELECT AVG(y) FROM " & dbTable)
End Sub

Public Sub AverageY As Double
	Return OverallAVG
End Sub

'Returns maximum if the chart is of rising character
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub isRisingY (StartPercent As Int, StopPercent As Int) As TrendResult
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	Dim StartPercent2 As Int = StartPercent * count / 100
	Dim StopPercent2 As Int = StopPercent * count / 100
	Dim Range As Int = StopPercent2 - StartPercent2
	If Range <= 0 Then 
		Log("Error: StartPercent must be smaller than StopPercent")
		'Return Null
	End If
	Private SubIntervals_qty As Int = Calc_SubIntervals(StartPercent2, StopPercent2) 'the source period is divided into this number parts [ i] (iterable) to compare them
	
	Dim Interval As Int = Floor(Range / SubIntervals_qty) + 1	'qty of analysing subintervals of the whole data set
	Dim operations As List
	operations.Initialize
	For i = StartPercent2 To StopPercent2 - Interval Step Interval
		Dim upto As Int = i + Interval
		'If upto > count - 1 Then upto = count - 1
		Dim req As String = "SELECT AVG(y) FROM " & dbTable & " WHERE id >= " & i & " AND id <= " & upto
		'Log(req)
		Dim avg As Double = mSQL.ExecQuerySingleResult(req)
		operations.Add(avg)
	Next
	
	Dim oper_qty As Int
	For i = 0 To operations.Size - 2
		Dim prev As Double = operations.Get(i)
		Dim nex As Double = operations.Get(i+1)
		If nex >= prev Then
			oper_qty = oper_qty + 1
		Else
			oper_qty = oper_qty - 1
		End If
	Next
	Private result As TrendResult
	result.Initialize
	If oper_qty >= operations.Size / 2 Then
		result.rising = True
	Else
		result.rising = False
	End If
	result.MaxY = MaxY(StartPercent, StopPercent)
	result.MaxX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " ORDER BY id DESC LIMIT 1")
	result.MinY = MinY(StartPercent, StopPercent)
	result.MinX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " WHERE id >= " & StartPercent2 & " ORDER BY id ASC LIMIT 1")
	result.avgX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " WHERE id >= " & (Range/2) & " LIMIT 1")
	result.avgY = AvgY(StartPercent, StopPercent)
	Return result
End Sub

'Returns minimum if the chart is of falling character
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub isFallingY (StartPercent As Int, StopPercent As Int) As TrendResult
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	Dim StartPercent2 As Int = StartPercent * count / 100
	Dim StopPercent2 As Int = StopPercent * count / 100
	Dim Range As Int = StopPercent2 - StartPercent2
	If Range <= 0 Then 
		Log("Error: StartPercent must be smaller than StopPercent")
		'Return Null
	End If
	Private SubIntervals_qty As Int = Calc_SubIntervals(StartPercent2, StopPercent2) 'the source period is divided into this number parts [ i] (iterable) to compare them
	Dim Interval As Int = Floor(Range / SubIntervals_qty) + 1	'qty of analysing subintervals of the whole data set
	Dim operations As List
	operations.Initialize
	For i = StartPercent2 To StopPercent2 - Interval Step Interval
		Dim upto As Int = i + Interval
		'If upto > count - 1 Then upto = count - 1
		Dim req As String = "SELECT AVG(y) FROM " & dbTable & " WHERE id >= " & i & " AND id <= " & upto
		'Log(req)
		Dim avg As Double = mSQL.ExecQuerySingleResult( req)
		operations.Add(avg)
	Next
	Dim oper_qty As Int
	For i = 0 To operations.Size - 2
		Dim prev As Double = operations.Get(i)
		Dim nex As Double = operations.Get(i+1)
		If nex <= prev Then
			oper_qty = oper_qty + 1
		Else
			oper_qty = oper_qty - 1
		End If
	Next
	Private result As TrendResult
	result.Initialize
	If oper_qty >= operations.Size / 2 Then
		result.falling = True
	Else
		result.falling = False
	End If

	result.MaxY = MaxY(StartPercent, StopPercent)
	result.MaxX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " ORDER BY id DESC LIMIT 1")
	result.MinY = MinY(StartPercent, StopPercent)
	result.MinX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " WHERE id >= " & StartPercent2 & " LIMIT 1")
	result.avgX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " WHERE id >= " & (Range/2) & " LIMIT 1")
	result.avgY = AvgY(StartPercent, StopPercent)
	Return result
End Sub

'Checks if the chart is wobbling character.
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub isWobblingY (StartPercent As Int, StopPercent As Int) As TrendResult
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	Dim StartPercent2 As Int = StartPercent * count / 100
	Dim StopPercent2 As Int = StopPercent * count / 100
	Dim Range As Int = StopPercent2 - StartPercent2
	If Range <= 0 Then 
		Log("Error: StartPercent must be smaller than StopPercent")
		'Return Null
	End If
	Private SubIntervals_qty As Int = Calc_SubIntervals(StartPercent2, StopPercent2) 'the source period is divided into this number parts [ i] (iterable) to compare them
	Dim Interval As Int = Floor(Range / SubIntervals_qty) + 1	'qty of analysing subintervals of the whole data set
	Dim operations As List
	operations.Initialize
	For i = StartPercent2 To StopPercent2 - Interval Step Interval
		Dim upto As Int = i + Interval
		'If upto > count - 1 Then upto = count - 1
		Dim req As String = "SELECT AVG(y) FROM " & dbTable & " WHERE id >= " & i & " AND id <= " & upto
		'Log(req)
		Dim avg As Double = mSQL.ExecQuerySingleResult( req)
		operations.Add(avg)
	Next
	Dim oper_qty As Int
	For i = 0 To operations.Size - 2
		Dim prev As Double = operations.Get(i)
		Dim nex As Double = operations.Get(i+1)
		If nex >= prev Then
			oper_qty = oper_qty + 1
		Else
			oper_qty = oper_qty - 1
		End If
	Next
	Private result As TrendResult
	result.Initialize
	If oper_qty <= operations.Size / 2 Then
		result.wobbling = True
	Else
		result.wobbling = False
	End If
	result.MaxY = MaxY(StartPercent, StopPercent)
	result.MaxX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " ORDER BY id DESC LIMIT 1")
	result.MinY = MinY(StartPercent, StopPercent)
	result.MinX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " WHERE id >= " & StartPercent2 & " LIMIT 1")
	result.avgX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " WHERE id >= " & (Range/2) & " LIMIT 1")
	result.avgY = AvgY(StartPercent, StopPercent)
	Return result
End Sub

'Returns max value if the chart has a maxumum
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub hasMaximum (StartPercent As Int, StopPercent As Int) As TrendResult
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	Dim StartPercent2 As Int = StartPercent * count / 100
	Dim StopPercent2 As Int = StopPercent * count / 100
	Dim Range As Int = StopPercent2 - StartPercent2
	If Range <= 0 Then
		Log("Error: StartPercent must be smaller than StopPercent")
		'Return Null
	End If

	Private LeftRising As TrendResult = isRisingY(StartPercent, StartPercent + (StopPercent - StartPercent)/2)
	Private RightFalling As TrendResult = isFallingY(StopPercent * 0.5, StopPercent)
	
	Private result As TrendResult
	result.Initialize
	result.MaxY = MaxY(StartPercent, StopPercent)
	result.MaxX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " ORDER BY id DESC LIMIT 1")
	result.MinY = MinY(StartPercent, StopPercent)
	result.MinX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " WHERE id >= " & StartPercent2 & " LIMIT 1")
	result.avgX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " WHERE id >= " & (Range/2) & " LIMIT 1")
	result.avgY = AvgY(StartPercent, StopPercent)
	
	If LeftRising.rising And RightFalling.falling Then
		'Max(LeftRising, MaxY(StartPercent, StopPercent))
		result.hasMaxY = True
	Else
		result.hasMaxY = False
	End If
	Return result
End Sub

'True if the chart has a minimum
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub hasMinimum (StartPercent As Int, StopPercent As Int) As TrendResult
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	Dim StartPercent2 As Int = StartPercent * count / 100
	Dim StopPercent2 As Int = StopPercent * count / 100
	Dim Range As Int = StopPercent2 - StartPercent2
	If Range <= 0 Then
		Log("Error: StartPercent must be smaller than StopPercent")
		'Return Null
	End If
	Private LeftFalling As TrendResult = isFallingY(StartPercent, StartPercent + (StopPercent - StartPercent)/2)
	Private RightRising As TrendResult = isRisingY(StopPercent * 0.5, StopPercent)
	
	Private result As TrendResult
	result.Initialize
	result.MaxY = MaxY(StartPercent, StopPercent)
	result.MaxX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " ORDER BY id DESC LIMIT 1")
	result.MinY = MinY(StartPercent, StopPercent)
	result.MinX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " WHERE id >= " & StartPercent2 & " LIMIT 1")
	result.avgX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " WHERE id >= " & (Range/2) & " LIMIT 1")
	result.avgY = AvgY(StartPercent, StopPercent)
	
	If LeftFalling.falling And RightRising.rising Then
		'Dim m As String = MinY(StartPercent, StopPercent)
		'Return Min(LeftFalling, m)
		result.hasMinY = True
	Else
		result.hasMinY = False
	End If
	Return result
End Sub

Private Sub Folder As String
	#if B4J
	Dim res As String = File.DirApp
	#else if B4A
	Dim res As String = DBUtils.GetDBFolder
	#end if
	Return res
End Sub

Public Sub IsInitialized As Boolean
	Return mInitialized
End Sub

Public Sub Close
	If mSQL.IsInitialized Then mSQL.Close
	File.Delete(Folder, dbName)
	mInitialized = False
End Sub

'Returns max of the range
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub MaxY (StartPercent As Int, StopPercent As Int) As String
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	Dim StartPercent2 As Int = StartPercent * count / 100
	Dim StopPercent2 As Int = StopPercent * count / 100
	Dim Range As Int = StopPercent2 - StartPercent2
	If Range <= 0 Then 
		Log("Error: StartPercent must be smaller than StopPercent")
		'Return Null
	End If

	Dim MaxValue As Float = mSQL.ExecQuerySingleResult("SELECT MAX(y) FROM " & dbTable & " WHERE id >= " & StartPercent2 & " AND id <= " & StopPercent2)
	Return MaxValue
End Sub

'Returns min of the range
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub MinY (StartPercent As Int, StopPercent As Int) As String
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	Dim StartPercent2 As Int = StartPercent * count / 100
	Dim StopPercent2 As Int = StopPercent * count / 100
	Dim Range As Int = StopPercent2 - StartPercent2
	If Range <= 0 Then 
		Log("Error: StartPercent must be smaller than StopPercent")
		'Return Null
	End If

	Dim MinValue As Float = mSQL.ExecQuerySingleResult( "SELECT MIN(y) FROM " & dbTable & " WHERE id >= " & StartPercent2 & " AND id <= " & StopPercent2)
	Return MinValue
End Sub

'Returns average value of the range
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub AvgY (StartPercent As Int, StopPercent As Int) As String
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	Dim StartPercent2 As Int = StartPercent * count / 100
	Dim StopPercent2 As Int = StopPercent * count / 100
	Dim Range As Int = StopPercent2 - StartPercent2
	If Range <= 0 Then 
		Log("Error: StartPercent must be smaller than StopPercent")
		'Return Null
	End If

	Dim AvgValue As Float = mSQL.ExecQuerySingleResult( "SELECT AVG(y) FROM " & dbTable & " WHERE id >= " & StartPercent2 & " AND id <= " & StopPercent2)
	Return AvgValue
End Sub

Private Sub Calc_SubIntervals (StartPercent As Int, StopPercent As Int) As Int
	Dim count  As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable & " WHERE id >= " & StartPercent & " AND id <= " & StopPercent)
	Private SubIntervals_qty As Int 'the source period is divided into this number parts [ i] (iterable) to compare them
	If count > 10 Then
		SubIntervals_qty = 10
	Else
		SubIntervals_qty = count / 2
	End If
	If SubIntervals_qty < 1 Then SubIntervals_qty = 1
	Return SubIntervals_qty
End Sub

'Checks if the chart is wobbling within +/- StableWithin/2 % value.
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub isStableY (StableWithin As Float, StartPercent As Int, StopPercent As Int) As TrendResult
	Dim count As Int = DBUtils.GetIntFromSQL(mSQL, dbTable, "SELECT count(*) FROM " & dbTable)	'data qty
	Dim StartPercent2 As Int = StartPercent * count / 100
	Dim StopPercent2 As Int = StopPercent * count / 100
	Dim Range As Int = StopPercent2 - StartPercent2
	If Range <= 0 Then 
		Log("Error: StartPercent must be smaller than StopPercent")
		'Return Null
	End If

	Private result As TrendResult
	result.Initialize
	result.MaxY = MaxY(StartPercent, StopPercent)
	result.MaxX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " ORDER BY id DESC LIMIT 1")
	result.MinY = MinY(StartPercent, StopPercent)
	result.MinX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " WHERE id >= " & StartPercent2 & " LIMIT 1")
	result.avgX = mSQL.ExecQuerySingleResult("SELECT x FROM " & dbTable & " WHERE id >= " & (Range/2) & " LIMIT 1")
	result.avgY = AvgY(StartPercent, StopPercent)
	
	Dim delta As Double = Abs(result.MaxY - result.MinY) / result.avgY * 100
'	Log("delta = " & delta & " %")

	If delta <= Abs(StableWithin)  Then
		result.stable = True
	Else
		result.stable = False
	End If
	result.StableWithin = StableWithin
	Return result
End Sub

'Returns an iconified string, with the automated checkings of the chart.
'Parts - analysis of pieces, whole range is splitted into chosen number of pieces.
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub AutoAnalysis1 (Parts As Int, StartPercent As Int, StopPercent As Int) As String
	Dim Range As Int = StopPercent - StartPercent
	If Range <= 0 Then
		Log("Error: StartPercent must be smaller than StopPercent")
		'Return Null
	End If
	Dim Interval As Int = Floor(Range / Parts) + 1	'qty of analysing subintervals of the whole data set
	'Dim operations As List
	'operations.Initialize
	Dim an As String
	For i = StartPercent To StopPercent - Interval  Step Interval
		Dim m As Map
		m.Initialize
		Dim upto As Int = i + Interval
		Dim rising As String = isRisingY(i, upto)
		Dim falling As String = isFallingY(i, upto)
		Dim wobbling As String = isWobblingY(i, upto)
		Dim hasmax As String = hasMaximum(i, upto)
		Dim hasmin As String = hasMinimum(i, upto)
		Dim stable100 As String = isStableY(100, i, upto)
		Dim stable50 As String = isStableY(50, i, upto)
		Dim stable20 As String = isStableY(20, i, upto)
		
		If rising <> "null" Then
			an = an & "↗"
		End If
		If falling <> "null" Then
			an = an & "↘"
		End If
		If wobbling <> "null" Then
			If stable20 <> "null" Then
				an = an & "↝↝↝"	'wobbling within a narrow range
			else if stable50 <> "null" Then
				an = an & "↝↝"	'wobbling within a middle size range
			else if stable100 <> "null" Then
				an = an & "↝"	'wobbling within a big range
			End If
		End If
		If hasmax <> "null" Then
			an = an & "↷"	'maximum
		End If
		If hasmin <> "null" Then
			an = an & "↺"	'minimum
		End If
		an = an & "."
	Next
	Return an
End Sub

'Automated checkings of the chart.
'Returns the map of approximated curve.
'Parts - analysis of pieces qty, whole range is splitted into chosen number of pieces.
'StartPercent, StopPercent: range of data, all data are when StartPercent = 0%, StartPercent = 100%
Public Sub AutoAnalysis2 (Parts As Int, StartPercent As Int, StopPercent As Int) As String
	Dim Range As Int = StopPercent - StartPercent
	If Range <= 0 Then
		Log("Error: StartPercent must be smaller than StopPercent")
		'Return Null
	End If
	Dim Interval As Int = Floor(Range / Parts) + 1	'qty of analysing subintervals of the whole data set
	Dim curve As Map
	curve.Initialize
	Dim an As String
	For i = StartPercent To StopPercent - Interval  Step Interval
		Dim m As Map
		m.Initialize
		Dim upto As Int = i + Interval
		Dim rising As String = isRisingY(i, upto)
		Dim falling As String = isFallingY(i, upto)
		Dim wobbling As String = isWobblingY(i, upto)
		Dim hasmax As String = hasMaximum(i, upto)
		Dim hasmin As String = hasMinimum(i, upto)
		Dim stable100 As String = isStableY(100, i, upto)
		Dim stable50 As String = isStableY(50, i, upto)
		Dim stable20 As String = isStableY(20, i, upto)
		
		If rising <> "null" Then
			an = an & "↗"
		End If
		If falling <> "null" Then
			an = an & "↘"
		End If
		If wobbling <> "null" Then
			If stable20 <> "null" Then
				an = an & "↝↝↝"	'wobbling within a narrow range
			else if stable50 <> "null" Then
				an = an & "↝↝"	'wobbling within a middle size range
			else if stable100 <> "null" Then
				an = an & "↝"	'wobbling within a big range
			End If
		End If
		If hasmax <> "null" Then
			an = an & "↷"	'maximum
		End If
		If hasmin <> "null" Then
			an = an & "↺"	'minimum
		End If
		an = an & "."
	Next
	Return an
End Sub